@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc 3084 2006-01-25 07:49:16Z fyodor $ */
+/* $Id: nmap.cc 3120 2006-02-07 07:15:32Z fyodor $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -480,7 +480,7 @@ int nmap_main(int argc, char *argv[]) {
 	o.setXSLStyleSheet(optarg);
       } else if (optcmp(long_options[option_index].name, "no-stylesheet") == 0) {
 	o.setXSLStyleSheet(NULL);
-      } else if (strcmp(long_options[option_index].name, "system-dns") == 0) {
+      } else if (optcmp(long_options[option_index].name, "system-dns") == 0) {
         o.mass_dns = false;
       } else if (optcmp(long_options[option_index].name, "dns-servers") == 0) {
         o.dns_servers = strdup(optarg);
