@@ -96,7 +96,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_error.cc,v 1.9 2004/08/29 09:12:03 fyodor Exp $ */
+/* $Id: nmap_error.cc 2751 2005-07-19 05:07:51Z fyodor $ */
 
 #include "nmap_error.h"
 
@@ -108,27 +108,29 @@
 extern void CloseLibs(void);
 #endif
 
-void fatal(char *fmt, ...) {
+void fatal(const char *fmt, ...) {
 va_list  ap;
 va_start(ap, fmt);
 fflush(stdout);
 vfprintf(stderr, fmt, ap);
 fprintf(stderr, "\nQUITTING!\n");
+fflush(stderr);
 va_end(ap);
 exit(1);
 }
 
-void error(char *fmt, ...) {
+void error(const char *fmt, ...) {
 va_list  ap;
 va_start(ap, fmt);
 fflush(stdout);
 vfprintf(stderr, fmt, ap);
 fprintf(stderr, "\n");
+fflush(stderr);
 va_end(ap);
 return;
 }
 
-void pfatal(char *err, ...) {
+void pfatal(const char *err, ...) {
 #ifdef WIN32
 	int lasterror =0;
 	char *errstr = NULL;
@@ -150,7 +152,7 @@ void pfatal(char *err, ...) {
 	exit(1);
 }
 
-void gh_perror(char *err, ...) {
+void gh_perror(const char *err, ...) {
 va_list  ap;va_start(ap, err);
 fflush(stdout);
 vfprintf(stderr, err, ap);
