@@ -98,12 +98,13 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: osscan.cc 2813 2005-08-11 05:07:01Z fyodor $ */
+/* $Id: osscan.cc 3039 2006-01-11 01:32:04Z fyodor $ */
 
 
 #include "osscan.h"
 #include "timing.h"
 #include "NmapOps.h"
+#include "tty.h"
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -118,6 +119,119 @@
 
 extern NmapOps o;
 
+/* Note that a sport of 0 really will (try to) use zero as the source
+   port rather than choosing a random one */
+struct udpprobeinfo *send_closedudp_probe(int sd, struct eth_nfo *eth,
+					  const struct in_addr *victim,
+					  u16 sport, u16 dport) {
+
+static struct udpprobeinfo upi;
+static int myttl = 0;
+static u8 patternbyte = 0;
+static u16 id = 0; 
+u8 packet[328]; /* 20 IP hdr + 8 UDP hdr + 300 data */
+struct ip *ip = (struct ip *) packet;
+udphdr_bsd *udp = (udphdr_bsd *) (packet + sizeof(struct ip));
+struct in_addr *source;
+int datalen = 300;
+unsigned char *data = packet + 28;
+unsigned short realcheck; /* the REAL checksum */
+int res;
+int decoy;
+struct pseudo_udp_hdr {
+  struct in_addr source;
+  struct in_addr dest;        
+  u8 zero;
+  u8 proto;        
+  u16 length;
+} *pseudo = (struct pseudo_udp_hdr *) ((char *)udp - 12) ;
+
+if (!patternbyte) patternbyte = (get_random_uint() % 60) + 65;
+memset(data, patternbyte, datalen);
+
+while(!id) id = get_random_uint();
+
+/* check that required fields are there and not too silly */
+if ( !victim || !dport || (!eth && sd < 0)) {
+  fprintf(stderr, "send_closedudp_probe: One or more of your parameters suck!\n");
+  return NULL;
+}
+
+if (!myttl)  myttl = (time(NULL) % 14) + 51;
+
+for(decoy=0; decoy < o.numdecoys; decoy++) {
+  source = &o.decoys[decoy];
+
+  memset((char *) packet, 0, sizeof(struct ip) + sizeof(udphdr_bsd));
+
+  udp->uh_sport = htons(sport);
+  udp->uh_dport = htons(dport);
+  udp->uh_ulen = htons(8 + datalen);
+
+  /* Now the pseudo header for checksuming */
+  pseudo->source.s_addr = source->s_addr;
+  pseudo->dest.s_addr = victim->s_addr;
+  pseudo->proto = IPPROTO_UDP;
+  pseudo->length = htons(sizeof(udphdr_bsd) + datalen);
+  
+  /* OK, now we should be able to compute a valid checksum */
+  realcheck = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ +
+		       datalen);
+#if STUPID_SOLARIS_CHECKSUM_BUG
+  udp->uh_sum = sizeof(udphdr_bsd) + datalen;
+#else
+  udp->uh_sum = realcheck;
+#endif
+
+  if ( o.badsum )
+    udp->uh_sum++;
+
+  /* Goodbye, pseudo header! */
+  memset(pseudo, 0, sizeof(*pseudo));
+  
+  /* Now for the ip header */
+  ip->ip_v = 4;
+  ip->ip_hl = 5;
+  ip->ip_len = htons(sizeof(struct ip) + sizeof(udphdr_bsd) + datalen);
+  ip->ip_id = id;
+  ip->ip_ttl = myttl;
+  ip->ip_p = IPPROTO_UDP;
+  ip->ip_src.s_addr = source->s_addr;
+  ip->ip_dst.s_addr= victim->s_addr;
+  
+  upi.ipck = in_cksum((unsigned short *)ip, sizeof(struct ip));
+#if HAVE_IP_IP_SUM
+  ip->ip_sum = upi.ipck;
+#endif
+  
+  /* OK, now if this is the real she-bang (ie not a decoy) then
+     we stick all the inph0 in our upi */
+  if (decoy == o.decoyturn) {   
+    upi.iptl = 28 + datalen;
+    upi.ipid = id;
+    upi.sport = sport;
+    upi.dport = dport;
+    upi.udpck = realcheck;
+    upi.udplen = 8 + datalen;
+    upi.patternbyte = patternbyte;
+    upi.target.s_addr = ip->ip_dst.s_addr;
+  }
+  if (TCPIP_DEBUGGING > 1) {
+    log_write(LOG_STDOUT, "Raw UDP packet creation completed!  Here it is:\n");
+    readudppacket(packet,1);
+  }
+  
+  if ((res = send_ip_packet(sd, eth, packet, ntohs(ip->ip_len))) == -1)
+    {
+      perror("send_ip_packet in send_closedupd_probe");
+      return NULL;
+    }
+}
+
+return &upi;
+}
+
+
 FingerPrint *get_fingerprint(Target *target, struct seq_info *si) {
 FingerPrint *FP = NULL, *FPtmp = NULL;
 FingerPrint *FPtests[9];
@@ -182,7 +296,7 @@ get_random_bytes(&sequence_base, sizeof(unsigned int));
   } else {
     /* Init our raw socket */
     if ((rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
-      pfatal("socket trobles in get_fingerprint");
+      pfatal("socket troubles in get_fingerprint");
     unblock_socket(rawsd);
     broadcast_socket(rawsd);
 #ifndef WIN32
@@ -390,7 +504,7 @@ if (o.verbose && openport != (unsigned long) -1)
      t1 = seq_send_times[seq_packets_sent];
      seq_packets_sent++;
    
-     /* Now we collect  the replies */
+     /* Now we collect the replies */
      while(si->responses < seq_packets_sent && !timeout) {
        
        if (seq_packets_sent == NUM_SEQ_SAMPLES)
@@ -929,7 +1043,7 @@ double compare_fingerprints(FingerPrint *referenceFP, FingerPrint *observedFP,
 }
 
 /* Takes a fingerprint and looks for matches inside reference_FPs[].
-   The results are stored in in FPR (which must point to an instantiated
+   The results are stored in FPR (which must point to an instantiated
    FingerPrintResults class) -- results will be reverse-sorted by
    accuracy.  No results below accuracy_threshhold will be included.
    The max matches returned is the maximum that fits in a
@@ -1170,9 +1284,11 @@ int bestaccidx;
  if (target->timedOut(NULL))
    return 1;
  
+o.scantype = OS_SCAN;
+
 #ifdef WIN32
   if (target->ifType() == devt_loopback) {
-    log_write(LOG_STDOUT, "Skipping OS Scan against %s because it doesn't work against your own machine (localhsot)\n", target->NameIP());
+    log_write(LOG_STDOUT, "Skipping OS Scan against %s because it doesn't work against your own machine (localhost)\n", target->NameIP());
     return 1;
   }
 #endif
@@ -1203,6 +1319,11 @@ int bestaccidx;
    if (target->timedOut(&now))
      return 1;
 
+   // Check if a status message is requested
+   if (keyWasPressed()) {
+      // Do nothing because the keyWasPressed Method prints out the basic status line
+   }
+
    target->FPR->FPs[itry] = get_fingerprint(target, &si[itry]); 
 
    match_fingerprint(target->FPR->FPs[itry], &FP_matches[itry], 
@@ -1706,114 +1827,6 @@ return AVs;
 }
 
 
-struct udpprobeinfo *send_closedudp_probe(int sd, struct eth_nfo *eth,
-					  const struct in_addr *victim,
-					  u16 sport, u16 dport) {
-
-static struct udpprobeinfo upi;
-static int myttl = 0;
-static u8 patternbyte = 0;
-static u16 id = 0; 
-u8 packet[328]; /* 20 IP hdr + 8 UDP hdr + 300 data */
-struct ip *ip = (struct ip *) packet;
-udphdr_bsd *udp = (udphdr_bsd *) (packet + sizeof(struct ip));
-struct in_addr *source;
-int datalen = 300;
-unsigned char *data = packet + 28;
-unsigned short realcheck; /* the REAL checksum */
-int res;
-int decoy;
-struct pseudo_udp_hdr {
-  struct in_addr source;
-  struct in_addr dest;        
-  u8 zero;
-  u8 proto;        
-  u16 length;
-} *pseudo = (struct pseudo_udp_hdr *) ((char *)udp - 12) ;
-
-if (!patternbyte) patternbyte = (get_random_uint() % 60) + 65;
-memset(data, patternbyte, datalen);
-
-while(!id) id = get_random_uint();
-
-/* check that required fields are there and not too silly */
-if ( !victim || !sport || !dport || (!eth && sd < 0)) {
-  fprintf(stderr, "send_closedudp_probe: One or more of your parameters suck!\n");
-  return NULL;
-}
-
-if (!myttl)  myttl = (time(NULL) % 14) + 51;
-
-for(decoy=0; decoy < o.numdecoys; decoy++) {
-  source = &o.decoys[decoy];
-
-  memset((char *) packet, 0, sizeof(struct ip) + sizeof(udphdr_bsd));
-
-  udp->uh_sport = htons(sport);
-  udp->uh_dport = htons(dport);
-  udp->uh_ulen = htons(8 + datalen);
-
-  /* Now the psuedo header for checksuming */
-  pseudo->source.s_addr = source->s_addr;
-  pseudo->dest.s_addr = victim->s_addr;
-  pseudo->proto = IPPROTO_UDP;
-  pseudo->length = htons(sizeof(udphdr_bsd) + datalen);
-  
-  /* OK, now we should be able to compute a valid checksum */
-  realcheck = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ +
-		       datalen);
-#if STUPID_SOLARIS_CHECKSUM_BUG
-  udp->uh_sum = sizeof(udphdr_bsd) + datalen;
-#else
-  udp->uh_sum = realcheck;
-#endif
-
-  /* Goodbye, pseudo header! */
-  memset(pseudo, 0, sizeof(*pseudo));
-  
-  /* Now for the ip header */
-  ip->ip_v = 4;
-  ip->ip_hl = 5;
-  ip->ip_len = htons(sizeof(struct ip) + sizeof(udphdr_bsd) + datalen);
-  ip->ip_id = id;
-  ip->ip_ttl = myttl;
-  ip->ip_p = IPPROTO_UDP;
-  ip->ip_src.s_addr = source->s_addr;
-  ip->ip_dst.s_addr= victim->s_addr;
-  
-  upi.ipck = in_cksum((unsigned short *)ip, sizeof(struct ip));
-#if HAVE_IP_IP_SUM
-  ip->ip_sum = upi.ipck;
-#endif
-  
-  /* OK, now if this is the real she-bang (ie not a decoy) then
-     we stick all the inph0 in our upi */
-  if (decoy == o.decoyturn) {   
-    upi.iptl = 28 + datalen;
-    upi.ipid = id;
-    upi.sport = sport;
-    upi.dport = dport;
-    upi.udpck = realcheck;
-    upi.udplen = 8 + datalen;
-    upi.patternbyte = patternbyte;
-    upi.target.s_addr = ip->ip_dst.s_addr;
-  }
-  if (TCPIP_DEBUGGING > 1) {
-    log_write(LOG_STDOUT, "Raw UDP packet creation completed!  Here it is:\n");
-    readudppacket(packet,1);
-  }
-  
-  if ((res = send_ip_packet(sd, eth, packet, ntohs(ip->ip_len))) == -1)
-    {
-      perror("send_ip_packet in send_closedupd_probe");
-      return NULL;
-    }
-}
-
-return &upi;
-
-}
-
 struct AVal *fingerprint_portunreach(struct ip *ip, struct udpprobeinfo *upi) {
 struct icmp *icmp;
 struct ip *ip2;
@@ -2012,7 +2025,7 @@ int ipid_sequence(int numSamples, u16 *ipids, int islocalhost) {
 	if (ipid_diffs[i] % 256 == 0) /* Stupid MS */
 	  ipid_diffs[i] -= 256;
 	else
-	  ipid_diffs[i]--; /* Because on localhost the RST sent back ues an IPID */
+	  ipid_diffs[i]--; /* Because on localhost the RST sent back use an IPID */
       }
     }
   }
