@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: portlist.cc 2813 2005-08-11 05:07:01Z fyodor $ */
+/* $Id: portlist.cc 2964 2005-12-05 01:00:03Z fyodor $ */
 
 
 #include "portlist.h"
@@ -178,8 +178,8 @@ static void populateFullVersionString(struct serviceDeductions *sd) {
 }
 
 
-// pass in an allocated struct serviceDeductions (don't wory about
-// initializing, and you don't have to free any inernal ptrs.  See the
+// pass in an allocated struct serviceDeductions (don't worry about
+// initializing, and you don't have to free any internal ptrs.  See the
 // serviceDeductions definition for the fields that are populated.
 // Returns 0 if at least a name is available.
 int Port::getServiceDeductions(struct serviceDeductions *sd) {
@@ -258,7 +258,7 @@ int Port::getServiceDeductions(struct serviceDeductions *sd) {
 // will be NULL if unavailable. Note that this function makes its
 // own copy of sname and product/version/extrainfo.  This function
 // also takes care of truncating the version strings to a
-// 'reasonable' length if neccessary, and cleaning up any unprinable
+// 'reasonable' length if neccessary, and cleaning up any unprintable
 // chars. (these tests are to avoid annoying DOS (or other) attacks
 // by malicious services).  The fingerprint should be NULL unless
 // one is available and the user should submit it.  tunnel must be
@@ -434,6 +434,7 @@ PortList::~PortList() {
 int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
   Port *current = NULL;
   map < u16, Port* > *portarray = NULL;  // This has to be a pointer so that we change the original and not a copy
+  map <u16, Port *>::iterator pt;
   char msg[128];
 
   assert(state < PORT_HIGHEST_STATE);
@@ -457,7 +458,7 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
     fatal("addPort: attempt to add port number %d with illegal state %d\n", portno, state);
 
   if (protocol == IPPROTO_TCP) {
-     portarray = &tcp_ports;
+    portarray = &tcp_ports;
   } else if (protocol == IPPROTO_UDP) {
     portarray = &udp_ports;
   } else if (protocol == IPPROTO_IP) {
@@ -465,10 +466,11 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
     portarray = &ip_prots;
   } else fatal("addPort: attempted port insertion with invalid protocol");
 
-  if (portarray->find(portno) == portarray->end()) {
+  pt = portarray->find(portno);
+  if (pt != portarray->end()) {
     /* We must discount our statistics from the old values.  Also warn
        if a complete duplicate */
-    current = (*portarray)[portno];    
+    current = pt->second;
     if (o.debugging && current->state == state && (!owner || !*owner)) {
       error("Duplicate port (%hu/%s)\n", portno, proto2ascii(protocol));
     } 
@@ -502,15 +504,6 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
     current->owner = strdup(owner);
   }
   
-  /*
-  printf("\nCurrent Port List\n");
-  
-  for(map<u16,Port*>::iterator iter = (*portarray).begin(); iter != (*portarray).end(); iter++)
-    printf("(%d %d) ", iter->first, iter->second->portno);
-  printf("\n");
-  */
-  
-
   return 0; /*success */
 }
 
@@ -547,7 +540,7 @@ int PortList::removePort(u16 portno, u8 protocol) {
 }
 
   /* Saves an identification string for the target containing these
-     ports (an IP addrss might be a good example, but set what you
+     ports (an IP address might be a good example, but set what you
      want).  Only used when printing new port updates.  Optional.  A
      copy is made. */
 void PortList::setIdStr(const char *id) {
@@ -561,15 +554,24 @@ void PortList::setIdStr(const char *id) {
 }
 
 Port *PortList::lookupPort(u16 portno, u8 protocol) {
+  map <u16, Port *>::iterator pt;
+  if (protocol == IPPROTO_TCP) {
+    pt = tcp_ports.find(portno);
+    if (pt != tcp_ports.end())
+      return pt->second;
+  }
 
-  if (protocol == IPPROTO_TCP)
-    return tcp_ports[portno];
+  else if (protocol == IPPROTO_UDP) {
+    pt = udp_ports.find(portno);
+    if (pt != udp_ports.end())
+      return pt->second;
+  }
 
-  if (protocol == IPPROTO_UDP)
-    return udp_ports[portno];
-
-  if (protocol == IPPROTO_IP)
-    return ip_prots[portno];
+  else if (protocol == IPPROTO_IP) {
+    pt = ip_prots.find(portno);
+    if (pt != ip_prots.end())
+      return pt->second;
+  }
 
   return NULL;
 }
